import os
import json
import getpass

from github import Github

from .notifier import notify, generate_message

USER_CREDENTIALS = os.path.expanduser('~') + '/.ghnotifier_config'
USER_DATA = os.path.expanduser('~') + '/.ghnotifier_data'

def create_user():

    # read/create a configuration file for user credentials
    try:
        with open(USER_CREDENTIALS, 'r') as f:
            credentials = json.load(f)

    except FileNotFoundError:
        with open(USER_CREDENTIALS, 'w') as f:
            USERNAME = input("Enter your GitHub username: ")
            PASSWORD = getpass.getpass(prompt="Enter your password: ")
            # store the credentials
            credentials = {
                    "USERNAME": USERNAME,
                    "PASSWORD": PASSWORD 
                    }
            json.dump(credentials, f)

    # create a data file if it does not already exist.
    try:
        with open(USER_DATA, 'r') as fp:
            pass

    except FileNotFoundError:
        with open(USER_DATA, 'w') as fp:
            # Schema for user data
            data = {
                    credentials["USERNAME"]:
                        {
                            "followers": 0,
                            "repos": []
                        }
                    }
            json.dump(data, fp)

    g = Github(credentials["USERNAME"], credentials["PASSWORD"])
    user = g.get_user()
    return user

def main_handler():
    user = create_user()
    # get user_data
    with open(USER_DATA, 'r') as fp:
        user_data = json.load(fp)[user.login]

    followers_status = handle_followers(user_data["followers"], user)
    user_data["followers"] = user.followers
    print("Execution of handle_followers is over.")
    print("Execution of handling repos is commencing.")
    for repo in user.get_repos():
        print("Handling repo: %s", repo.name)
        for i, old_repo in enumerate(user_data["repos"]):
            if repo.name in old_repo:
                updated_repo = handle_repo(old_repo, repo)
                user_data["repos"][i] = updated_repo
                break           
        else:
            new_repo = {
                        "name": repo.name,
                        "forks": 0,
                        "stars": 0,
                        "watchers": 0
                        }
        
            updated_repo = handle_repo(new_repo, repo)
            user_data["repos"].append(updated_repo) 
        print("Handling %s over", repo.name)

    print("Writing data")
    with open(USER_DATA, 'w') as fp:
        json.dump(user_data, fp)


def logic(current_data, previous_data, context=None):
    delta = current_data - previous_data
    if delta == 0:
        return 0

    elif delta > 0:
        message = generate_message(delta, context, 1)

    else:
        message = generate_message(delta, context, -1)

    notify(message)
    return 1

# handle user's followers updates
def handle_followers(user_followers, user):
    previous_followers = user_followers
    user_followers = current_followers = user.followers
    context = ("follow", user.login)

    followers_handle = []
    number_of_new_followers = current_followers - previous_followers
    if(number_of_new_followers == 0): # no change in following
        return 0

    elif(number_of_new_followers > 0): # new followers
        for follower in user.get_followers()[previous_followers:]:
            followers_handle.append(follower.login)

        for f in followers_handle:
            message = generate_message(f, context, 1)
            notify(message)

        return 1

    else: # people unfollowed
        message = generate_message(number_of_new_followers, context, -1)
        notify(message)
        return -1

# handle user's repo updates
def handle_repo(user_repo, repo):
        previous_forks = user_repo["forks"]
        previous_stars = user_repo["stars"]
        previous_watchers = user_repo["watchers"]
       
        user_repo["forks"] = current_forks = repo.forks_count
        user_repo["stars"] = current_stars = repo.stargazers_count
        user_repo["watchers"] = current_watchers = repo.watchers_count

        name = repo.name
        logic(current_stars, previous_stars, context=("starr", name)) # past tense
        logic(current_watchers, previous_watchers, context=("watch", name))
        logic(current_forks, previous_forks, context=("fork", name))

        return user_repo



import requests
from bs4 import BeautifulSoup


def user_kitchen(handle):
    """Function to make soup for a user.

    Parameters
    ==========

    handle : str
        The GitHub handle of the user

    Returns
    =======

    soup : BeautifulSoup
    """
    base_url = 'https://github.com/'
    user = base_url + handle
    r = requests.get(user)
    data = r.text
    soup = BeautifulSoup(data, 'lxml')
    return soup


def detail_scrapper(soup, attribute):
    """Function to scrape user and profile details from the soup."""
    tagset = soup.find_all(attrs=attribute)
    data = []
    for tag in tagset:
        data.append(tag.text)

    return data


class User(object):
    """The class to represent a GitHub user.

    Each user is identified by his unique GitHub handle.
    """
    details_map = {'fullname': {'class': 'vcard-fullname'},
                   'bio': {'class': 'user-profile-bio'},
                   'company': {'aria-label': 'Organization'},
                   'location': {'aria-label': 'Home location'},
                   'email': {'aria-label': 'Email'},
                   'membership': {'aria-label': 'Member since'}}

    def __init__(self, handle):
        self.handle = str(handle)
        self.soup = user_kitchen(handle)
        self.user_details()
        self.profile_details()

    def user_details(self):
        value_map = {}
        for detail, attribute in details_map.items():
            value_map[detail] = detail_scrapper(self.soup, attribute)[0]

        self.fullname = value_map['fullname']
        self.bio = value_map['bio']
        self.company = value_map['company']
        self.location = value_map['location']
        self.email = value_map['email']
        self.membership = value_map['membership']

    def profile_details(self):
        attribute = {'class': 'vcard-stat-count d-block'}
        self.followers, self.starred, self.following = detail_scrapper(self.soup, attribute)
        self.organizations = _get_organizations()

    def _get_organizations(self):
        bar = self.soup.find_all(attrs={'class': 'clearfix'})[0].find_all('a')
        organizations = []
        for i in bar:
            org = i.get('aria-label')
            organizations.append(org)
        return organizations

    def profile_pic(self):
        link = self.soup.find_all('img', class_='avatar rounded-2')[0].get('src')
        return link



from __future__ import division
import sys
from math import ceil
import requests

base_url = 'https://api.github.com/search/users'


def user_search_filter(filterkey):
    """Filter for user search pages.

    Parameters
    ==========

    filterkey : String
        The keyword to filter the results out based on some criteria.

        Currently, the following criterias are supported:
        1. Number of followers := 'Most followers' or 'Fewest followers'
        2. Number of repositories := 'Most repositories' or 'Fewest repositories'
        3. Relative date of joining := 'Most recently joined' or 'Least recently joined'
        If nothing is passed, the result of 'Best Match' is returned.

    Returns
    =======

    dict
        A dictionary of search parameters based on `filterkey`.

    """
    user_search_parameters = ('follow', 'repo', 'join')
    measure_table = {'desc': ('most', 'high', 'large'), 'asc': ('least', 'less', 'few')}
    timeline_table = {'desc': ('recent', 'new'), 'asc': ('last', 'old')}

    search_params = {}

    if any(parameter in filterkey for parameter in user_search_parameters):
        if 'join' in filterkey:
            search_params['s'] = 'joined'
            table = timeline_table

        else:
            table = measure_table
            if 'follow' in filterkey:
                search_params['s'] = 'followers'
            else:
                search_params['s'] = 'repositories'

        if any(word in filterkey for word in table['asc']):
            search_params['o'] = 'asc'

        else:
            search_params['o'] = 'desc'  # the default behaviour returns "Most followers".

    else:
        raise ValueError("Enter a valid filterkey. Read the documentation "
                         "to know about the supported search parameters.")

    return search_params


def search_location(location, keyword=None, max_users=10, params={}):
    """Location based search.

    Parameters
    ==========

    location : String
        The name of the concerned location.

    keyword : String
        The filter for search a criteria.

    max_users : int
        The maximum number of users of interest.
        Using a number larger than 100 can lead to loss of results due to
        making "Too Many Requests" [HTTP Status Code: 429].

    params : dict
        Any specified parameters.

    Returns
    =======

    list
        A list of handles of the users at a particular location based on the
        specified criterias(if any).

    """
    params['q'] = 'location:' + location
    params['per_page'] = 100
    if keyword is not None:
        new_params = user_search_filter(keyword)
        params.update(new_params)

    response = requests.get(base_url, params=params)
    content = response.json()
    total_users = content['total_count']
    number_of_users = min(max_users, total_users)
    number_of_pages = int(number_of_users / 100)
    
    users = _get_users(response.json()['items'])
    for i in range(2, number_of_pages):
        params['page'] = i
        r = requests.get(base_url, params=params)
        new_users = _get_users(r.json()['items'])
        users.append(new_users)
    
    return users[:number_of_users]


def _get_users(user_list):
    users = []
    for user in user_list:
        users.append(user["login"])

    return users



import sys
import subprocess


def generate_message(protagonist, context, status):
    if status == 1:
        return "{} {}ed {}".format(protagonist, *context)
    elif status == 0:
        return "{} users {}ed {}".format(protagonist, *context)
    elif status == -1:
        return "{} users un{}ed {}".format(protagonist, *context)
    else:
        print("Enter valid status code for generating message")


def notify(message):
    """ Notifies the logged in user with an update message using the notifer application based on the Operating System.

    For MacOS : terminal-notifier
        Linux : notify-send

    """
    title = "GitHub Notification"
    platform = sys.platform
    if platform == 'darwin':
        # test if terminal-notifier is present on the system or not
        status = subprocess.run(["which", "terminal-notifier"]).returncode
        if status == 0:
            notifier = "terminal-notifier"
            subprocess.run([notifier, "-title", "GitHub Notification", "-message",
                            message, "-timeout", "10"])

        else:
            print("Kindly install terminal-notifier for MacOS.")
            return -1

    elif platform == 'linux':
        notifier = "notify-send"
        subprocess.run([notifier, "GitHub Notification", message])

    # look into other cases
    else:
        print("Your system is not supported yet.")
        return -1

    return 1
